@@ -19,7 +19,7 @@
 import time
 import copy
 from contextlib import suppress
-from unittest import mock, skip
+from unittest import mock
 
 from qiskit.providers.ibmq.apiconstants import API_JOB_FINAL_STATES, ApiJobStatus
 from qiskit.test.mock import FakeQobj
@@ -385,7 +385,6 @@ class TestIBMQJobStates(JobTestCase):
                     else:
                         self.assertFalse(self._current_api.job_get.called)
 
-    @skip("Skip until marshmallow removed from result, so correct exception is raised.")
     def test_no_kind_job(self):
         """Test a job without the kind field."""
         job = self.run_with_api(NoKindJobAPI())
@@ -393,6 +392,12 @@ class TestIBMQJobStates(JobTestCase):
             job.result()
         self.assertIsNone(job.qobj())
 
+    def test_transpiling_status(self):
+        """Test transpiling job state."""
+        job = self.run_with_api(TranspilingStatusAPI())
+        time.sleep(0.2)
+        self.assertEqual(job.status(), JobStatus.INITIALIZING)
+
     def run_with_api(self, api):
         """Creates a new ``IBMQJob`` running with the provided API object."""
         backend = IBMQBackend(mock.Mock(), mock.Mock(), mock.Mock(), api_client=api)
@@ -663,3 +668,12 @@ class NoKindJobAPI(BaseFakeAPI):
 
     def job_result(self, job_id, *_args, **_kwargs):
         return self.no_kind_response
+
+
+class TranspilingStatusAPI(BaseFakeAPI):
+    """Class for emulating an API with transpiling status codes."""
+
+    _job_status = [
+        {'status': 'TRANSPILING'},
+        {'status': 'TRANSPILED'}
+    ]
